
/***********************************************************************************
  Copyright 2008 Texas Instruments Incorporated. All rights reserved.

  IMPORTANT: Your use of this Software is limited to those specific rights
  granted under the terms of a software license agreement between the user
  who downloaded the software, his/her employer (which must be your employer)
  and Texas Instruments Incorporated (the "License").  You may not use this
  Software unless you agree to abide by the terms of the License. The License
  limits your use, and you acknowledge, that the Software may not be modified,
  copied or distributed unless embedded on a Texas Instruments microcontroller
  or used solely and exclusively in conjunction with a Texas Instruments radio
  frequency transceiver, which is integrated into your product.  Other than for
  the foregoing purpose, you may not use, reproduce, copy, prepare derivative
  works of, modify, distribute, perform, display or sell this Software and/or
  its documentation for any purpose.

  YOU FURTHER ACKNOWLEDGE AND AGREE THAT THE SOFTWARE AND DOCUMENTATION ARE
  PROVIDED AS IS WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED,
  INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, TITLE,
  NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL
  TEXAS INSTRUMENTS OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER CONTRACT,
  NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR OTHER
  LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
  INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE
  OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT
  OF SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
  (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.

  Should you have any questions regarding your right to use this Software,
  contact Texas Instruments Incorporated at www.TI.com.
***********************************************************************************/

/***********************************************************************************
  Filename: simpleapi.c

  Description:  This example show how to construct the different simple api frame formats
                used with the CC2480 and reads the response.

  A button press selects the command to output the message response on the virtual
  serial port. Configure the PC HyperTerminal for baudrate 9600 bps, 8 data bits
  no parity, and 1 stop bit.

***********************************************************************************/

/***********************************************************************************
* INCLUDES
*/
#include <hal_led.h>
#include <hal_board.h>
#include <hal_assert.h>
#include <hal_uart.h>
#include <util.h>

#include "cc2480.h"
#include "cc2480_rpc_spi.h"
#include "sapi.h"

/***********************************************************************************
* LOCAL VARIABLES
*/
static uint8  type;
static uint16 poll;
static uint16 qpoll;
static uint16 rpoll;

static uint16 TEST_POLL_VALUE  = 0x1122;
static uint16 TEST_QPOLL_VALUE = 0xABCD;
static uint16 TEST_RPOLL_VALUE = 0xEDC5;

/***********************************************************************************
* LOCAL FUNCTIONS
*/

static void appPrintMenu(void);


/******************************************************************************
* @fn          appPrintMenu
*
* @brief       Prints menu on UART
*
* @param       none
*
* @return      none
*/
static void appPrintMenu(void)
{
    printStr("------------------------------\n");
    printStr("CC2480 Application Example\n");
    printStr("Simple API, SAPI\n");
    printStr("------------------------------\n\n");
}




/***********************************************************************************
* @fn          main
*
* @brief       This is the main entry of the "simpleapi" application
*
* @param       none
*
* @return      none
*/
void main (void)
{
    uint8 value;
    uint8  rtrnCmdId;
    uint8  rtrnCmdType;

    // Initalise board peripherals
    halBoardInit();

    // Indicate that device is powered
    halLedSet(1);

    // Initalise UART
    halUartInit(HAL_UART_BAUDRATE_9600, 0);

    // Print Logo Hyperterminal
    appPrintMenu();

    // Clear all CC2480 Configurations and restore to default values using HARD reset
    // Get system reset response
    cc2480HardReset();

    // Puts MCU in endless loop
    while(TRUE) {
        // Wait for user to press S1
        while (!HAL_BUTTON_1_PUSHED());

        halMcuWaitMs(350);

        // Make sure NV RAM is cleared
        value = ZCD_STARTOPT_CLEAR_CONFIG;
        zb_WriteConfiguration(ZCD_NV_STARTUP_OPTION, 1, &value);

        // Extract the command ID of the response
        rtrnCmdType = hostMsgBuf[1];
        rtrnCmdId   = hostMsgBuf[2];

        // Output to serial port Write Configuration Response
        // Please see CC2480 2.4GHz ZigBee Processor User Guide Page 22 & 23
        // for command description
        printKvpHex8bit("Response Length", hostMsgBuf[0]);
        printKvpHex    ("Write Config Response Cmd ID", BUILD_UINT16( rtrnCmdId, rtrnCmdType ));
        printKvpHex8bit("Status ", hostMsgBuf[3]);

        // Any reconfiguration of this parameter will require a reset to take effect.
        zb_SystemReset();

        // Wait for user to press S1
        while (!HAL_BUTTON_1_PUSHED());
        halMcuWaitMs(350);

        zb_ReadConfiguration(ZCD_NV_LOGICAL_TYPE);

        // Extract the command ID of the response
        rtrnCmdType = hostMsgBuf[1];
        rtrnCmdId   = hostMsgBuf[2];

        // Output to serial port Read Configuration Response
        // Please see CC2480 2.4GHz ZigBee Processor User Guide Page 22 & 23
        // for command description
        printKvpHex8bit("Response Length", hostMsgBuf[0]);
        printKvpHex    ("Read Config Response Cmd ID", BUILD_UINT16( rtrnCmdId, rtrnCmdType ));
        printKvpHex8bit("Status ", hostMsgBuf[3]);
        printKvpHex8bit("Configuration ID ", hostMsgBuf[4]);
        printKvpHex8bit("Length of Value Buffer", hostMsgBuf[5]);
        printKvpHex8bit("Configuration Value", hostMsgBuf[6]);

        type  = *(uint8 *)&(hostMsgBuf[6]);
        if (type == ZG_DEVICETYPE_COORDINATOR)
        {
          printKvpHex8bit("Coordinator", type);
        } else {
           halLedSet(2);
           halLedClear(1);
        }

        // Wait for user to press S1
        while (!HAL_BUTTON_1_PUSHED());
        halMcuWaitMs(350);

        // Write data to NVRAM
        // Change ZigBee Logical Device Type to End Device
        // Change End Device Poll Rates
        type  = ZG_DEVICETYPE_ENDDEVICE;
        poll  = ++TEST_POLL_VALUE;
        qpoll = ++TEST_QPOLL_VALUE;
        rpoll = ++TEST_RPOLL_VALUE;

        zb_WriteConfiguration(ZCD_NV_LOGICAL_TYPE, 1, &type);
        zb_WriteConfiguration(ZCD_NV_POLL_RATE, 2, &poll);
        zb_WriteConfiguration(ZCD_NV_QUEUED_POLL_RATE, 2, &qpoll);
        zb_WriteConfiguration(ZCD_NV_RESPONSE_POLL_RATE, 2, &rpoll);

        // Read NVRAM
        // Change ZigBee Logical Device Type to End Device
        // Change End Device Poll Rates

        zb_ReadConfiguration(ZCD_NV_LOGICAL_TYPE);
        type  = *(uint8 *)&(hostMsgBuf[6]);
        zb_ReadConfiguration(ZCD_NV_POLL_RATE);
        poll  = (hostMsgBuf[7] << 8) | hostMsgBuf[6];
        zb_ReadConfiguration(ZCD_NV_QUEUED_POLL_RATE);
        qpoll = (hostMsgBuf[7] << 8) | hostMsgBuf[6];
        zb_ReadConfiguration(ZCD_NV_RESPONSE_POLL_RATE);
        rpoll = (hostMsgBuf[7] << 8) | hostMsgBuf[6];

        // Verify Content of NV RAM
        // Please see CC2480 2.4GHz ZigBee Processor User Guide Page 24 & 25
        // for command description
        if ((type   == ZG_DEVICETYPE_ENDDEVICE) &&
            (poll   == TEST_POLL_VALUE) &&
            (qpoll  == TEST_QPOLL_VALUE) &&
            (rpoll  == TEST_RPOLL_VALUE))
        {
           // Output to serial port Read Configuration Response verified content
           printKvpHex8bit("End Device", type);
           printKvpHex("Poll Rate", poll);
           printKvpHex("Queued Poll Rate", qpoll);
           printKvpHex("Response Poll Rate", rpoll);
        }
    }

}
