
/***********************************************************************************
    Filename:     assy_ez430_rf2480.c

    Description:  Defines connections between the MSPF2232 and the CC2480

    Copyright (c) 2008 by Texas Instruments, Inc.
    All Rights Reserved.  Permission to use, reproduce, copy, prepare
    derivative works, modify, distribute, perform, display or sell this
    software and/or its documentation for any purpose is prohibited
    without the express written consent of Texas Instruments, Inc.
***********************************************************************************/

/***********************************************************************************
* INCLUDES
*/
#include "assy_ez430_rf2480.h"


/***********************************************************************************
* FUNCTIONS
*/
static void halAssyRadioIfcInit(uint32 divider);
static void halAssyRadioReset(void);



/***********************************************************************************
* @fn          halRadioSpiInit
*
* @brief       Initalise Radio SPI interface
*
* @param       none
*
* @return      none
*/
static void halAssyRadioIfcInit(uint32 divider)
{
    /* Clear MRDY and Disable SPI interface */
    ZACCEL_SPI_END();
    /* Set optional CCZACC configuration pins */
    ZACCEL_CFG0_OPIN(1);
    ZACCEL_CFG1_OPIN(1);

    /* Put state machine in reset */
    UCB0CTL1 |= UCSWRST;

    UCB0BR0 = LOWORD(divider);
    UCB0BR1 = HIWORD(divider);

    /* 8-bit data SPI Master 4-pin mode, with SMCLK as clock source */
    UCB0CTL0  = UCCKPH + UCMSB + UCMST + UCSYNC;

    UCB0CTL1 |= UCSSEL_2;

    /* Select SPI functionality on DIO pins for MOSI, MISO, & CLK. */

    /* MOSI */
    MCU_IO_PERIPHERAL(3,1);
    MCU_IO_SET_HIGH(3,1);
    /* MISO */
    MCU_IO_PERIPHERAL(3,2);
    /* SCLK */
    MCU_IO_PERIPHERAL(3,3);
    MCU_IO_SET_HIGH(3,3);

    /* Release for operation */
    UCB0CTL1 &= ~UCSWRST;
}


/***********************************************************************************
* @fn      halAssyRadioReset
*
* @brief   Controlled hard reset of CC2480. No read out of reset indication
*
* @param   none
*
* @return  none
*/
static void halAssyRadioReset(void)
{
    ZACCEL_RESET();

    while(ZACCEL_SRDY_IN() != 0);

    /* In case the Z-Accel is still doing a power-up CRC check, toggle MRDY to signal to
     * write the result and thereby bypass all future power-up CRC checks.
     */
    ZACCEL_MRDY_SET();
    ZACCEL_MRDY_CLR();
}


/***********************************************************************************
* @fn      halAssyInit
*
* @brief   Initialize interfaces between radio and MCU
*
* @param   none
*
* @return  none
*/
void halAssyInit(void)
{
    halAssyRadioIfcInit(3);
    halAssyRadioReset();
}


/***********************************************************************************
* @fn      halAssyIoInit
*
* @brief   Initialize IO pins on assembly
*
* @param   none
*
* @return  none
*/
void halAssyIoInit(void)
{
    HAL_LED_DIR_OUT_1();
    HAL_LED_DIR_OUT_2();

    /* P2.0 is the light sensor input to ADC10-0.
    * P2.1 is the light sensor input enable - set high to read ADC, then low to save power.
    */
    MCU_IO_PERIPHERAL(HAL_BOARD_IO_LS_LEVEL_PORT,HAL_BOARD_IO_LS_LEVEL_PIN);
    HAL_LS_DIR_OUT();

    MCU_IO_INPUT(HAL_BOARD_IO_BTN_1_PORT, HAL_BOARD_IO_BTN_1_PIN, MCU_IO_PULLUP);

    /* P2.6 is a DI to receive SRDY (Slave Ready on RPC protocol */
    MCU_IO_INPUT(HAL_BOARD_IO_ZACC_SRDY_PORT, HAL_BOARD_IO_ZACC_SRDY_PIN, MCU_IO_PULLUP);

   /* P3.0 is a DO to control SSn (Slave Select Not = active low.)
    * P3.6 is a DO to control MRDY (Master Ready on RPC protocol.)
    */
    MCU_IO_OUTPUT(HAL_BOARD_IO_ZACC_SS_PORT, HAL_BOARD_IO_ZACC_SS_PIN, 1);
    MCU_IO_OUTPUT(HAL_BOARD_IO_ZACC_MRDY_PORT, HAL_BOARD_IO_ZACC_MRDY_PIN, 1);

    /* Keep board in reset */
    MCU_IO_OUTPUT(HAL_BOARD_IO_ZACC_RST_PORT,  HAL_BOARD_IO_ZACC_RST_PIN, 0);

    MCU_IO_INPUT(HAL_BOARD_IO_ZACC_DBGDC_PORT, HAL_BOARD_IO_ZACC_DBGDC_PIN, MCU_IO_PULLUP);
    MCU_IO_INPUT(HAL_BOARD_IO_ZACC_DBGDD_PORT, HAL_BOARD_IO_ZACC_DBGDD_PIN, MCU_IO_PULLUP);

    /* P4.0 is a DO to control Cfg0 (Slave Configuration Switch 0.)
    * - Set high configures slave to use 32-kHz crystal installed and used.
    * - Set low configures slave to use internal oscillator.
    * P4.1 is a DO to control Cfg1 (Slave Configuration Switch 1.)
    * - Set high configures slave to use SPI transport.
    * - Set low configures slave to use UART transport.
    */
    MCU_IO_OUTPUT(HAL_BOARD_IO_ZACC_CFG0_PORT, HAL_BOARD_IO_ZACC_CFG0_PIN,1);
    MCU_IO_OUTPUT(HAL_BOARD_IO_ZACC_CFG1_PORT, HAL_BOARD_IO_ZACC_CFG1_PIN,1);

}

