/***********************************************************************************
  Filename:     cc2480.c

  Description:  This file implements basic functions for accessing CC2480

***********************************************************************************/

/***********************************************************************************
* INCLUDES
*/
#include "cc2480.h"
#include "cc2480_rpc_spi.h"


/***********************************************************************************
* CONSTANTS AND DEFINES
*/


/***********************************************************************************
* GLOBAL VARIABLES
*/


/***********************************************************************************
* @fn      cc2480HardReset
*
* @brief   Hardware reset of chip (Hard Reset)
*
* @param   none
*
* @return  uint16 - System Reset Indication ID
*/
uint16 cc2480HardReset(void)
{
    uint8  rtrnCmdId;
    uint8  rtrnCmdType;

    /* Reset CC2480 */
    ZACCEL_RESET();

     /* Wait for SRDY to go low */
    while (ZACCEL_SRDY_IN() != 0);

    /* Get SYS_RESET_IND from CC2480 */
    cc2480RpcPoll(hostMsgBuf);

    /* Extract the command ID of the response */
    rtrnCmdType = hostMsgBuf[1];
    rtrnCmdId   = hostMsgBuf[2];

    return (BUILD_UINT16( rtrnCmdId, rtrnCmdType ));
}


/***********************************************************************************
* @fn      cc2480SoftReset
*
* @brief   Software reset chip (Soft Reset)
*
* @param   none
*
* @return  uint16 - System Reset Indication ID
*/
uint16 cc2480SoftReset(void)
{
    uint8 *pBuf = hostMsgBuf;
    uint8  rtrnCmdId;
    uint8  rtrnCmdType;

    /* Construct the SYS_RESET_REQ command issued by the application processor */
    *(pBuf+RPC_POS_LEN)   = 0x01;
    *(pBuf+RPC_POS_CMD0)  = RPC_CMDSUBSYS_SYS | RPC_CMD_AREQ;
    *(pBuf+RPC_POS_CMD1)  = CMDID_SYS_RESET_REQ;
    *(pBuf+RPC_POS_PARAM) = 0x00;

    /* Send Soft Reset over SPI bus */
    cc2480Rpc(hostMsgBuf);

    /* Extract the command ID of the response */
    rtrnCmdType = hostMsgBuf[1];
    rtrnCmdId   = hostMsgBuf[2];

    return (BUILD_UINT16( rtrnCmdId, rtrnCmdType ));
}


/***********************************************************************************
* @fn      cc2480GetChipId
*
* @brief   Get chip id
*
* @param   none
*
* @return  uint8 - The chip ID
*/
uint8 cc2480GetChipId(void)
{
    uint8 *pBuf = hostMsgBuf;

    *(pBuf+RPC_POS_LEN) = 0;
    *(pBuf+RPC_POS_CMD0) = RPC_CMDSUBSYS_SYS | RPC_CMD_SREQ;
    *(pBuf+RPC_POS_CMD1) = CMDID_SYS_VERSION;

    cc2480Rpc(hostMsgBuf);

    return (hostMsgBuf[4]);
}


/***********************************************************************************
* @fn      cc2480GetIEEEAddress
*
* @brief   Get IEEE address
*
* @param   uint8' buffer - where to write the IEEE Address
*
* @return  uint8 - 0
*/
uint8 cc2480GetIEEEAddress(uint8* buffer)
{
    uint8 *pBuf = hostMsgBuf;
    uint8 i;

    *(pBuf+RPC_POS_LEN) = 0x01;
    *(pBuf+RPC_POS_CMD0) = RPC_CMDSUBSYS_SAPI | RPC_CMD_SREQ;
    *(pBuf+RPC_POS_CMD1) = CMDID_ZB_GET_DEVICE_INFO;

    /* Parameter 1: Device IEEE address */
    *(pBuf+RPC_POS_PARAM) = 0x01;

    cc2480Rpc(hostMsgBuf);

    /* Write the IEEE address to the specified buffer */
    for (i = 0; i < 8; i++) {
        buffer[i] = hostMsgBuf[4 + i];
    }

    return 0;
}


/***********************************************************************************
* @fn      cc2480Write
*
* @brief   Write to TX buffer; Read RX buffer
*
* @param   uint8* data   - buffer to write and read
*          uint8  length - number of bytes
*
* @return  uint8  - Always 0
*/
uint8 cc2480Write(uint8 *data, uint8 length)
{
    while(length--) {
        ZACCEL_SPI_TX(*data);
        ZACCEL_SPI_WAIT_RXRDY();
        *data = ZACCEL_SPI_RX();
        data++;
    }
    return(0);
}


/***********************************************************************************
* @fn      cc2480Read
*
* @brief   Read RX buffer
*
* @param   uint8* data   - data buffer. This must be allocated by caller.
*          uint8  length - number of bytes
*
* @return  uint8  - Always 0
*/
uint8 cc2480Read(uint8 *data, uint8 length)
{
    while (length--) {
        ZACCEL_SPI_TX(0x00);
        ZACCEL_SPI_WAIT_RXRDY();
        *data = ZACCEL_SPI_RX();
        data++;
    }
    return(0);
}


/***********************************************************************************
  Copyright 2007 Texas Instruments Incorporated. All rights reserved.

  IMPORTANT: Your use of this Software is limited to those specific rights
  granted under the terms of a software license agreement between the user
  who downloaded the software, his/her employer (which must be your employer)
  and Texas Instruments Incorporated (the "License").  You may not use this
  Software unless you agree to abide by the terms of the License. The License
  limits your use, and you acknowledge, that the Software may not be modified,
  copied or distributed unless embedded on a Texas Instruments microcontroller
  or used solely and exclusively in conjunction with a Texas Instruments radio
  frequency transceiver, which is integrated into your product.  Other than for
  the foregoing purpose, you may not use, reproduce, copy, prepare derivative
  works of, modify, distribute, perform, display or sell this Software and/or
  its documentation for any purpose.

  YOU FURTHER ACKNOWLEDGE AND AGREE THAT THE SOFTWARE AND DOCUMENTATION ARE
  PROVIDED AS IS WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED,
  INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, TITLE,
  NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL
  TEXAS INSTRUMENTS OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER CONTRACT,
  NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR OTHER
  LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
  INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE
  OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT
  OF SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
  (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.

  Should you have any questions regarding your right to use this Software,
  contact Texas Instruments Incorporated at www.TI.com.
***********************************************************************************/
