/***********************************************************************************
    Filename:     rpc_spi_cc2480.c

	Description:  RPC SPI master implementation for MSP430

************************************************************************************/


/***********************************************************************************
* INCLUDES
*/
#include "hal_types.h"
#include "hal_defs.h"
#include "hal_mcu.h"
#include "hal_board.h"

#include "cc2480.h"
#include "cc2480_rpc_spi.h"



/***********************************************************************************
* CONSTANTS AND DEFINES
*/
#define CC2480_PACKET_OVERHEAD_SIZE 3


/***********************************************************************************
* TYPEDEFS
*/


/***********************************************************************************
* LOCAL VARIABLES
*/


/***********************************************************************************
* GLOBAL VARIABLES
*/
uint8 hostMsgBuf[RPC_MAX_BUF];


/***********************************************************************************
* LOCAL FUNCTIONS
*/
static uint8 rpcSpiAREQ(uint8 *pBuf);
static uint8 rpcSpiSREQ(uint8 *pBuf);



/**************************************************************************************************
 * @fn          cc2480Rpc
 *
 * @brief       This function effects the requested RPC transaction across the configured medium.
 *
 * input parameters
 *
 * @param       pBuf - Pointer to the RPC buffer.
 *
 * output parameters
 *
 * None.
 *
 * @return      TRUE if RPC transaction with slave succeeded; FALSE otherwise.
 **************************************************************************************************
 */
uint8 cc2480Rpc(uint8 *pBuf)
{
  uint8 rtrn = 0;

  if (RPC_CMD_SREQ == (*(pBuf+RPC_POS_CMD0) & (RPC_CMD_SREQ | RPC_CMD_AREQ)))
  {
    if (rpcSpiSREQ(pBuf))
    {
      rtrn = TRUE;
    }
  }
  else
  {
    rtrn = rpcSpiAREQ(pBuf);
  }
  return rtrn;
}


/**************************************************************************************************
 * @fn          cc2480RpcPoll
 *
 * @brief       This function polls the NP slave according to the RPC protocol for POLL.
 *
 * input parameters
 *
 * @param       pBuf - Pointer to the buffer to Tx a POLL across the SPI and then receive the AREQ.
 *
 * output parameters
 *
 * @param       pBuf - Pointer to the buffer to Tx a POLL across the SPI and then receive the AREQ.
 *
 * @return      TRUE if RPC transaction with slave succeeded; FALSE otherwise.
 **************************************************************************************************
 */
uint8 cc2480RpcPoll(uint8 *pBuf)
{

  *(pBuf+RPC_POS_LEN)  = 0;                /* POLL command has zero data bytes. */
  *(pBuf+RPC_POS_CMD0) = RPC_CMD_POLL;     /* POLL command MSB. */
  *(pBuf+RPC_POS_CMD1) = 0;                /* POLL command LSB. */

  return rpcSpiSREQ(pBuf);
}


/**************************************************************************************************
 * @fn          spiAREQ
 *
 * @brief       This function effects an asynchronous transaction with the NP slave
 *              according to the RPC protocol for the AREQ.
 *
 * input parameters
 *
 * @param       pBuf - Pointer to the buffer to Tx across the SPI.
 *
 * output parameters
 *
 * None.
 *
 * @return      TRUE if RPC transaction with slave succeeded; FALSE otherwise.
 **************************************************************************************************
 */
static uint8 rpcSpiAREQ(uint8 *pBuf)
{

  ZACCEL_SPI_BEGIN();

  while (ZACCEL_SRDY_IN() != 0);

  cc2480Write(pBuf,(*pBuf+CC2480_PACKET_OVERHEAD_SIZE));

  /* Wait for the slave to signal that it is ready. */
  while (ZACCEL_SRDY_IN() == 0);

  ZACCEL_SPI_END();

  return RPC_SUCCESS;
}


/**************************************************************************************************
 * @fn          rpcSpiSREQ
 *
 * @brief       This function effects a synchronous transaction with the NP slave
 *              according to the RPC protocol for the SREQ.
 *
 * input parameters
 *
 * @param       pBuf - Pointer to the buffer to Tx across the SPI.
 *
 * output parameters
 *
 * @param       pBuf - Pointer to the data received across the SPI.
 *
 * @return      TRUE if RPC transaction with slave succeeded; FALSE otherwise.
 **************************************************************************************************
 */
static uint8 rpcSpiSREQ(uint8 *pBuf)
{
    volatile uint16 abort = 0;

    ZACCEL_SPI_BEGIN();

    /* Slave aborts a stuck RPC transaction by resetting, so it will come up with SRDY active low
     * in order to report the reset message. Therefore, the host can never get stuck here.
     */
    while (ZACCEL_SRDY_IN() != 0);

    cc2480Write(pBuf,(*pBuf+CC2480_PACKET_OVERHEAD_SIZE));

  /* Now setup the POLL command in the buffer which has just been transmitted and which will now
   * be used to receive the SRSP.
   */

    *(pBuf+RPC_POS_LEN) = 0;                  /* POLL command has zero data bytes. */
    *(pBuf+RPC_POS_CMD0) = RPC_CMD_POLL;     /* POLL command MSB. */
    *(pBuf+RPC_POS_CMD1) = 0;                /* POLL command LSB. */

    /* Wait for the slave to signal that it is ready. */
   while (ZACCEL_SRDY_IN() == 0);

    // Send POLL cmd
    cc2480Write(pBuf,CC2480_PACKET_OVERHEAD_SIZE);

  // Receive the rest of the slave's message if it is longer than the POLL.
  if (*pBuf != 0)
  {
    cc2480Write(pBuf+CC2480_PACKET_OVERHEAD_SIZE, *pBuf);
  }

   ZACCEL_SPI_END();

   return TRUE;
}


/***********************************************************************************
  Copyright 2007 Texas Instruments Incorporated. All rights reserved.

  IMPORTANT: Your use of this Software is limited to those specific rights
  granted under the terms of a software license agreement between the user
  who downloaded the software, his/her employer (which must be your employer)
  and Texas Instruments Incorporated (the "License").  You may not use this
  Software unless you agree to abide by the terms of the License. The License
  limits your use, and you acknowledge, that the Software may not be modified,
  copied or distributed unless embedded on a Texas Instruments microcontroller
  or used solely and exclusively in conjunction with a Texas Instruments radio
  frequency transceiver, which is integrated into your product.  Other than for
  the foregoing purpose, you may not use, reproduce, copy, prepare derivative
  works of, modify, distribute, perform, display or sell this Software and/or
  its documentation for any purpose.

  YOU FURTHER ACKNOWLEDGE AND AGREE THAT THE SOFTWARE AND DOCUMENTATION ARE
  PROVIDED AS IS WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED,
  INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, TITLE,
  NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL
  TEXAS INSTRUMENTS OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER CONTRACT,
  NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR OTHER
  LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
  INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE
  OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT
  OF SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
  (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.

  Should you have any questions regarding your right to use this Software,
  contact Texas Instruments Incorporated at www.TI.com.
***********************************************************************************/
