
/***********************************************************************************
    Filename:     hal_joystick.c

    Description:  HAL joystick control

    Copyright (c) 2006 by Texas Instruments, Inc.
    All Rights Reserved.  Permission to use, reproduce, copy, prepare
    derivative works, modify, distribute, perform, display or sell this
    software and/or its documentation for any purpose is prohibited
    without the express written consent of Texas Instruments, Inc.
***********************************************************************************/

/***********************************************************************************
* INCLUDES
*/
#include "hal_button.h"
#include "hal_digio.h"
#include "hal_board.h"
#include "hal_int.h"


/***********************************************************************************
* CONSTANTS
*/


/***********************************************************************************
* LOCAL VARIABLES
*/
static ISR_FUNC_PTR button_isr_tbl[HAL_BUTTON_MAX_EVENTS] = {0};
static const digioConfig pinButtonPush = {HAL_BOARD_IO_BTN_1_PORT,
                                          HAL_BOARD_IO_BTN_1_PIN,
                                            BV(HAL_BOARD_IO_BTN_1_PIN ),
                                            HAL_DIGIO_INPUT, 0};

static void halButtonPushISR(void);

/***********************************************************************************
* @fn      halJoystickInit
*
* @brief   Configure joystick move interrupt
*
* @param   none
*
* @return  none
*/
void halButtonInit(void)
{
    // Discrete input
    MCU_IO_INPUT(HAL_BOARD_IO_BTN_1_PORT,  HAL_BOARD_IO_BTN_1_PIN, MCU_IO_TRISTATE);

    halDigioConfig(&pinButtonPush);

    halDigioIntSetEdge(&pinButtonPush, HAL_DIGIO_INT_RISING_EDGE);
    halDigioIntConnect(&pinButtonPush, &halButtonPushISR);
    halDigioIntEnable(&pinButtonPush);
}


/***********************************************************************************
* @fn      halJoystickIntConnect
*
* @brief   Connect isr for joystick move interrupt. The parameter event tells for
*          which joystick direction this isr should be called.
*
* @param   event - Joystick direction
*          func - Pointer to function to connect
*
* @return  none
*/
uint8 halButtonIntConnect(uint8 event, ISR_FUNC_PTR func)
{
    istate_t key;
    HAL_INT_LOCK(key);
    switch(event) {
    case HAL_BUTTON_PUSHED:
        button_isr_tbl[HAL_BUTTON_PUSHED] = func;
        break;
    default:
        HAL_INT_UNLOCK(key); return FAILED;
    }
    HAL_INT_UNLOCK(key);
    return SUCCESS;
}


/***********************************************************************************
* @fn      halJoystickIntEnable
*
* @brief   Enable joystick move interrupt
*
* @param   event - Joystick direction
*
* @return  none
*/
uint8 halButtonIntEnable(uint8 event)
{
    // Ignore event parameter
    halDigioIntEnable(&pinButtonPush);
    return SUCCESS;
}


/***********************************************************************************
* @fn      halJoystickIntDisable
*
* @brief   Disable joystick move interrupt
*
* @param   none
*
* @return  none
*/
void halButtonIntDisable(void)
{
    halDigioIntDisable(&pinButtonPush);
}


/***********************************************************************************
* @fn      halJoystickMoveISR
*
* @brief   Interrupt service routine for joystick move interrupt
*
* @param   none
*
* @return  none
*/
static void halButtonPushISR(void)
{
    // Take a snapshot of the ports connected to joystick inputs
    uint8 direction = P1IN;
    uint8 pushed    = BUTTON_1_PUSHED();

    if (pushed)
    {
        if(button_isr_tbl[HAL_BUTTON_PUSHED] != 0)
            (*button_isr_tbl[ HAL_BUTTON_PUSHED])();
    }
    else
    {
        if(button_isr_tbl[HAL_BUTTON_INVALID]  != 0)
            (*button_isr_tbl[HAL_BUTTON_INVALID])();
    }

}

