
/***********************************************************************************
    Filename:     hal_uart.c

    Description:  hal UART library

    Copyright (c) 2006 by Texas Instruments, Inc.
    All Rights Reserved.  Permission to use, reproduce, copy, prepare
    derivative works, modify, distribute, perform, display or sell this
    software and/or its documentation for any purpose is prohibited
    without the express written consent of Texas Instruments, Inc.
***********************************************************************************/

/***********************************************************************************
* INCLUDES
*/
#include "hal_types.h"
#include "hal_uart.h"
#include "hal_board.h"
#include "util_buffer.h"
#include "hal_int.h"
#include "hal_mcu.h"

/***********************************************************************************
* LOCAL VARIABLES
*/
static ringBuf_t rbRxBuf;

/***********************************************************************************
* @fn      halUartRxIntEnable
*
* @brief   Enable UART RX interrupt
*
* @param   none
*
* @return  none
*/
static void halUartRxIntEnable(void)
{
    IE2 |= UCA0RXIE;
}


/***********************************************************************************
* @fn      halUARTRxIntDisable
*
* @brief   Disable UART RX interrupt
*
* @param   none
*
* @return  none
*/
static void halUartRxIntDisable(void)
{
    IE2 &= ~UCA0RXIE;
}



/***********************************************************************************
* @fn      halUartRxGetByte
*
* @brief   Read byte from UART RX buffer
*
* @param   none
*
* @return  none
*/
static uint8 halUartRxGetByte(void)
{
    return UCA0RXBUF;
}


/***********************************************************************************
* @fn      halUartInit
*
* @brief   Initalise UART. Supported baudrates are: 19220, 38400, 57600 and 115200
*
* @param   uint8 baudrate
*          uint8 options - this parameter is ignored
*
* @return  none
*/
void halUartInit(uint8 baudrate, uint8 options)
{
    // Fixed settings: 9600, 8N1

    UCA0CTL1 = UCSWRST;          // Keep USART1 in reset state

    UCA0CTL1 |= UCSSEL_3;        // Set clock source SMCLK
    UCA0CTL0  = 0;               // 8-N-1, LSB

    UCA0BR0  = 0x34;             // 8MHz 9600 with 16x oversampling
    UCA0BR1  = 0x00;             // 8MHz 9600 with 16x oversampling
    UCA0MCTL = UCOS16 | UCBRF0;  // 8MHz 9600 with 16x oversampling

    P3SEL |= BIT4;               // P3.4 = USART1 TXD
    P3SEL |= BIT5;               // P3.5 = USART1 RXD

    UCA0CTL1 &= ~UCSWRST;        // Initialize USART1 state machine
/*
    P3SEL |= 0x30;                            // P3.4,5 = USCI_A0 TXD/RXD
    UCA0CTL1 |= UCSSEL_2;                      // SMCLK
    UCA0BR0 = 0x41;                           // 9600 from 8Mhz
    UCA0BR1 = 0x3;
    UCA0MCTL = UCBRS_2;
    UCA0CTL1 &= ~UCSWRST;                     // **Initialize USCI state machine**
*/
    // Enable RX interrupt
    halUartRxIntEnable();
}


/***********************************************************************************
* @fn      halUartWrite
*
* @brief   Write data buffer to UART
*
* @param   uint8* buf - buffer with data to write
*          uint16 length - number of bytes to write
*
* @return  uint16 - number of bytes written
*/
uint16 halUartWrite(const uint8* buf, uint16 length)
{
    uint16 i;
    for(i = 0; i < length; i++)
    {
        while(!(IFG2 & UCA0TXIFG));    // Wait for TX buffer ready to receive new byte
        UCA0TXBUF = buf[i];            // Output character
    }
    return (i+1);
}


/***********************************************************************************
* @fn      halUartRead
*
* @brief   Read data from UART Rx buffer
*
* @param   uint8* buf - buffer with data to read in to
*          uint16 length - number of bytes to read
*
* @return  none
*/
uint16 halUartRead(uint8* buf, uint16 length)
{
    return bufGet(&rbRxBuf, (uint8 *)buf, length);
}


/***********************************************************************************
* @fn      halUartGetNumRxBytes
*
* @brief   Returns number of bytes in RX buffer
*
* @param   none
*
* @return  uint8
*/
uint16 halUartGetNumRxBytes(void)
{
    return bufNumBytes(&rbRxBuf);
}


/***********************************************************************************
* @fn      halUartEnableRxFlow
*
* @brief   Signal ready/not ready to receive characters on UART
*
* @param   uint8 enable - TRUE to signal ready to receive on UART
*                         FALSE to signal not ready to receive on UART
*
* @return  none
*/
void halUartEnableRxFlow(uint8 enable)
{
    // Enable RX flow
    if(enable) {
        HAL_RTS_CLR();
    }
    // Disable RX flow
    else {
        HAL_RTS_SET();
    }
}


/***********************************************************************************
* @fn      usart1Rx_ISR
*
* @brief   ISR framework for the USCI A0/B0 Receive component
*
* @param   none
*
* @return  none
*/
#pragma vector=USCIAB0RX_VECTOR
__interrupt void usciA0Rx_ISR(void)
{
    uint8 ch = halUartRxGetByte();
    bufPut(&rbRxBuf,&ch,1);
    __low_power_mode_off_on_exit();
}
