/*********************************************************************
    
    Filename:       util_console.c

    Description:    Various console functions

    Notes:		

*********************************************************************/

#ifdef WIN32
/*********************************************************************
 * INCLUDES
 */
#include "hal_cc_uart.h"
#include "util_console.h"
#include "string.h"


/*********************************************************************
 * CONSTANTS
 */

/*********************************************************************
 * MACROS
 */

/*********************************************************************
 * TYPEDEFS
 */

/*********************************************************************
 * Global data
 */

static uint8 fdConsole= FD_UART;

 /*********************************************************************
 * FUNCTIONS
 */

/*********************************************************************
* @fn      setStdout
* @brief   Redirect the standard output stream
* @param   fd - console file descriptor (UART or remote).
* @return  none
*/
void setStdout(uint8 fd)
{
    fdConsole= fd;
}

/*********************************************************************
* @fn      printChar
* @brief   Print a character on the console.
* @param   c - character to print
* @return  none
*/
void printChar(char c)
{
    if (fdConsole==FD_UART) {
        uartPrintByte(c);
    } else {
        // TBD
    }
}

/*********************************************************************
* @fn      printStr
* @brief   Print a zero-terminated string on the console
* @param   sz - string to send to the console
* @return  none
*/
void printStr(char FAR *sz)
{

    if (fdConsole==FD_UART) {
       uartPrintStr(sz);
    } else {
        // TBD
    }
}

/*********************************************************************
 * @fn      printStrCode
 * @brief   Print a string store in code memory.
 * @param   none
 * @return  none
 */
void printStrCode(const char CODE *p)
{
    while (*p!='\0') {
        printChar(*p++);
    }
}

/*********************************************************************
* @fn      printStrW
* @brief   Print a string padded with blanks.
* @param   sz - string to display
* @param   len - total length of the string
* @return  none
*/
void printStrW(char FAR *sz, uint8 nBytes)
{
    printStr((char FAR *)sz);
    if (strlen(sz)<nBytes) {
        nBytes-= strlen(sz);
        while(--nBytes>0)
            printChar(' ');
    }

}

/*********************************************************************
* @fn      printLine
* @brief   Print a delimiter line.
* @param   none
* @return  none
*/
void printLine(uint8 c, uint8 n)
{
    while(n-->0)
        printChar(c);
    printChar('\n');

}

/*********************************************************************
* @fn      printKvpHex
* @brief   Print a Key-Value pair hexadecimal
* @param   szKey - the value to be displayed
* @param   v - the value to be displayed
* @return  none
*/
void printKvpHex(char FAR *szKey, uint16 v)
{
    printStrW(szKey, KVP_ALIGN);
    printStr(": ");
    printHex(v,16);
    printChar('\n');
}

/*********************************************************************
* @fn      printKvpInt
* @brief   Print a Key-Value pair hexadeciaml.
* @param   szKey - the value to be displayed
* @param   v - the value to be displayed
* @return  none
*/
void printKvpInt(char FAR *szKey, uint32 v)
{
    printStrW(szKey, KVP_ALIGN);
    printStr(": ");
    printInt(v);
    printChar('\n');
}

/*********************************************************************
* @fn      printTime
* @brief   Print time in HH:MM:SS:ms format
* @param   nMsec - milliseconds
* @return  none
*/
void printTime(uint32 nMsec)
{
    uint32 nSec;
    uint16 nMin;
    uint16 nHour;

    nSec= nMsec / 1000;
    nMin= (uint16)nSec / 60;
    nHour= nMin / 60;

    printStr("Time: ");
    printInt(nHour);
    printStr(":");

    nMin= nMin % 60;
    if (nMin<10)
        printStr("0");
    printInt(nMin);
    printStr(":");

    nSec= nSec % 60;
    if (nSec<10)
        printStr("0");
    printInt(nSec);
    printStr(".");

    nMsec= nMsec % 1000;
    if (nMsec<100) {
        printStr("0");
        if (nMsec<10) {
            printStr("0");
        }
    }
    printInt(nMsec);
    printChar('\n');
}


/*********************************************************************
* @fn      printInt
* @brief   Print a 32-bit integer as decimal.
* @param   v - the value to be displayed
* @return  none
*/
void printInt(uint32 v)
{
    uint32 iDiv;
    uint8 bLeadingZero;

    // Initialise
    iDiv= 1000000000;
    bLeadingZero= 1;

    // For each digit
    while (iDiv>0) {
        uint8 vDigit;

        // Current digit
        vDigit= (uint8)(v / iDiv);

        // First digit to be printed
        if (vDigit>0) {	
            bLeadingZero= 0;
        }

        if (!bLeadingZero) {
            printChar( (char)('0' + vDigit) );
        }

        // Next digit
        v= v % iDiv;
        iDiv= iDiv/10;
    }

    // Special case zero
    if (bLeadingZero) {
        printChar('0');
    }

}

/*********************************************************************
* @fn      printHex
* @brief   Print an integer as hexadeciaml.
* @param   v - the value to be displayed
* @param   s - size (8,16,32) | bit 7 to omit 'h' at the end
* @return  none
*/
void printHex(uint32 v, uint8 s)
{
    char buf[16];
    uint8 w;

    w= (s&~NO_HEX_INDICATION)>>2;        // String length

    if (s&NO_HEX_INDICATION) {
        buf[w]= '\0';
    } else {
        buf[w]= 'h';       // HEX
        buf[w+1]= '\0';    // String terminator
    }

    // Convert number
    do {
        w--;
        buf[w]= "0123456789ABCDEF"[v&0xF];
        v>>= 4;
    } while(w>0);

    // Print
    printStr(buf);

}

#endif